#!/usr/bin/env python3
"""
Advanced usage example for SemanticPen Python SDK
Demonstrates advanced features like request builder, custom polling, and complex workflows
"""

import os
import sys
import time
from semanticpen import (
    SemanticPenClient,
    SemanticPenConfig,
    PollingConfig,
    ArticleGenerationRequest
)


def demonstrate_request_builder(client):
    """Demonstrate the request builder pattern"""
    print("🏗️ Request Builder Pattern Example")
    print("-" * 40)
    
    try:
        # Create a complex request using builder pattern
        builder = client.create_request_builder()
        
        print("   Building complex article request...")
        
        # Build request with fluent API
        result = (builder
            .keyword("Advanced Python design patterns")
            .gpt_version("gpt-4")
            .project_name("Advanced Python Series")
            .preset("technical-deep-dive")
            .parameter("tone", "professional")
            .parameter("target_audience", "senior developers")
            .parameter("word_count", 2000)
            .parameter("include_code_examples", True)
            .generate())
        
        print(f"   ✅ Built and executed complex request")
        print(f"      Article ID: {result.article_id or result.article_ids[0]}")
        print(f"      Project ID: {result.project_id}")
        
        # Demonstrate generate and wait with builder
        print("\n   Building request with generate_and_wait...")
        
        builder2 = client.create_request_builder()
        article = (builder2
            .keyword("Python async/await best practices")
            .gpt_version("gpt-4")
            .project_name("Async Python Guide")
            .personalization("technical-writer")
            .parameter("include_examples", True)
            .generate_and_wait(
                polling_config=PollingConfig(interval=5000, max_attempts=20),
                callbacks={
                    "on_progress": lambda status, attempt, max_attempts: 
                        print(f"      Polling: {status['status']} (attempt {attempt}/{max_attempts})")
                }
            ))
        
        if isinstance(article, list):
            article = article[0]
        
        print(f"   ✅ Article completed: {article.title}")
        print(f"      Content length: {len(article.article_html or '')} characters")
        
    except Exception as e:
        print(f"   ❌ Builder pattern error: {e}")


def demonstrate_custom_polling(client):
    """Demonstrate custom polling configurations"""
    print("\n⏱️ Custom Polling Configuration Example")
    print("-" * 45)
    
    try:
        # Generate article first
        result = client.generate_article(
            "Custom polling example topic",
            project_name="Polling Demo"
        )
        article_id = result['articleId']
        
        print(f"   Generated article: {article_id}")
        
        # Custom polling with callbacks
        print("   Setting up custom polling with callbacks...")
        
        def on_progress(status, attempt, max_attempts):
            progress = (attempt / max_attempts) * 100
            print(f"      📊 Progress: {progress:.1f}% - Status: {status['status']}")
        
        def on_complete(article):
            print(f"      🎉 Completed: {article.title}")
        
        def on_error(error):
            print(f"      ❌ Error occurred: {error}")
        
        def on_timeout(error):
            print(f"      ⏰ Timeout: {error}")
        
        # Custom polling configuration
        custom_polling = PollingConfig(
            interval=3000,        # 3 seconds
            max_attempts=20,      # 1 minute total
            backoff_multiplier=1.2  # 20% increase each time
        )
        
        callbacks = {
            "on_progress": on_progress,
            "on_complete": on_complete,
            "on_error": on_error,
            "on_timeout": on_timeout
        }
        
        # Wait with custom configuration
        start_time = time.time()
        article = client.wait_for_article(
            article_id,
            polling_config=custom_polling,
            callbacks=callbacks
        )
        wait_time = time.time() - start_time
        
        print(f"   ✅ Custom polling completed in {wait_time:.2f} seconds")
        print(f"      Title: {article.title}")
        
    except Exception as e:
        print(f"   ❌ Custom polling error: {e}")


def demonstrate_bulk_workflows(client):
    """Demonstrate advanced bulk operation workflows"""
    print("\n📦 Advanced Bulk Workflows Example")
    print("-" * 40)
    
    try:
        # Define article series
        series_topics = [
            "Python metaclasses explained",
            "Decorator patterns in Python",
            "Context managers deep dive",
            "Generator functions and yield",
            "Asyncio event loop internals"
        ]
        
        print(f"   Planning article series: {len(series_topics)} articles")
        
        # Custom generation options
        generation_options = {
            "gpt_version": "gpt-4",
            "project_name": "Advanced Python Concepts",
            "additional_params": {
                "style": "tutorial",
                "difficulty": "advanced",
                "include_code": True
            }
        }
        
        # Custom polling for bulk operations
        bulk_polling = PollingConfig(
            interval=10000,  # Check every 10 seconds
            max_attempts=30,  # 5 minutes total
            backoff_multiplier=1.1
        )
        
        # Progress tracking
        def bulk_progress(completed, total, new_articles):
            percentage = (completed / total) * 100
            print(f"      📈 Series Progress: {completed}/{total} ({percentage:.1f}%)")
            for article in new_articles:
                print(f"         ✓ {article.title}")
        
        # Execute bulk workflow
        print("   Executing bulk generation and wait workflow...")
        start_time = time.time()
        
        workflow_result = client.generate_bulk_and_wait(
            series_topics,
            generation_options=generation_options,
            polling_config=bulk_polling,
            on_progress=bulk_progress
        )
        
        total_time = time.time() - start_time
        
        print(f"   ✅ Bulk workflow completed in {total_time:.2f} seconds")
        print(f"      ✓ Successful: {len(workflow_result['successful'])}")
        print(f"      ✗ Failed: {len(workflow_result['failed'])}")
        
        # Analyze results
        if workflow_result['successful']:
            print(f"\n   📊 Success Analysis:")
            total_words = 0
            for article in workflow_result['successful']:
                word_count = len((article.article_html or '').split())
                total_words += word_count
                print(f"      📰 {article.title}: {word_count} words")
            
            avg_words = total_words / len(workflow_result['successful'])
            print(f"      📈 Average article length: {avg_words:.0f} words")
            print(f"      📊 Total content generated: {total_words} words")
        
        if workflow_result['failed']:
            print(f"\n   ❌ Failure Analysis:")
            for failure in workflow_result['failed']:
                print(f"      ✗ {failure['keyword']}: {failure['error']}")
    
    except Exception as e:
        print(f"   ❌ Bulk workflow error: {e}")


def demonstrate_advanced_monitoring(client):
    """Demonstrate advanced monitoring and status tracking"""
    print("\n📊 Advanced Monitoring Example")
    print("-" * 35)
    
    try:
        # Generate several articles for monitoring
        print("   Creating articles for monitoring demo...")
        
        monitoring_topics = [
            "Real-time system monitoring",
            "Performance metrics tracking",
            "Error logging strategies"
        ]
        
        # Generate articles
        generation_result = client.generate_articles(monitoring_topics)
        
        if generation_result.successful:
            article_ids = [item['article_id'] for item in generation_result.successful]
            
            print(f"   📝 Generated {len(article_ids)} articles for monitoring")
            
            # Monitor progress over time
            print("   🔍 Starting monitoring loop...")
            
            monitoring_rounds = 5
            for round_num in range(monitoring_rounds):
                print(f"\n      Round {round_num + 1}/{monitoring_rounds}:")
                
                # Get bulk status
                status_result = client.get_multiple_status(article_ids)
                
                print(f"         📊 Status Overview:")
                print(f"            ✅ Completed: {len(status_result['completed'])}")
                print(f"            ⏳ Pending: {len(status_result['pending'])}")
                print(f"            ❌ Failed: {len(status_result['failed'])}")
                
                # Detailed status for each article
                for status in status_result['pending']:
                    print(f"            ⏳ {status['id']}: {status['status']}")
                
                for status in status_result['completed']:
                    print(f"            ✅ {status['id']}: {status['status']}")
                
                for status in status_result['failed']:
                    print(f"            ❌ {status['id']}: {status.get('errorMessage', 'Unknown error')}")
                
                # Break if all completed
                if len(status_result['completed']) + len(status_result['failed']) == len(article_ids):
                    print(f"         🎉 All articles processed!")
                    break
                
                # Wait before next check
                if round_num < monitoring_rounds - 1:
                    print(f"         ⏳ Waiting 10 seconds for next check...")
                    time.sleep(10)
        
    except Exception as e:
        print(f"   ❌ Monitoring error: {e}")


def demonstrate_configuration_patterns(client):
    """Demonstrate different configuration patterns"""
    print("\n⚙️ Configuration Patterns Example")
    print("-" * 38)
    
    try:
        # Get current configuration
        current_config = client.get_config()
        print(f"   📋 Current Configuration:")
        print(f"      Base URL: {current_config.base_url}")
        print(f"      Timeout: {current_config.timeout}ms")
        print(f"      Debug: {current_config.debug}")
        
        # Test different timeout configurations
        print(f"\n   🕐 Testing different timeout configurations...")
        
        # Short timeout configuration
        quick_config = SemanticPenConfig(
            api_key=current_config.api_key,
            base_url=current_config.base_url,
            timeout=5000,  # 5 seconds
            debug=True
        )
        
        quick_client = SemanticPenClient.create_with_config(quick_config)
        print(f"      ⚡ Created quick client (5s timeout)")
        
        # Long timeout configuration  
        patient_config = SemanticPenConfig(
            api_key=current_config.api_key,
            base_url=current_config.base_url,
            timeout=60000,  # 60 seconds
            debug=False
        )
        
        patient_client = SemanticPenClient.create_with_config(patient_config)
        print(f"      🐌 Created patient client (60s timeout)")
        
        # Test connection with different clients
        if quick_client.test_connection():
            print(f"      ✅ Quick client connection successful")
        
        if patient_client.test_connection():
            print(f"      ✅ Patient client connection successful")
        
    except Exception as e:
        print(f"   ❌ Configuration error: {e}")


def main():
    """Main advanced usage demonstration"""
    print("🚀 SemanticPen Python SDK - Advanced Usage Example")
    print("=" * 60)
    
    # Get API key
    api_key = os.getenv("SEMANTICPEN_API_KEY")
    
    if not api_key:
        print("⚠️  Please set SEMANTICPEN_API_KEY environment variable")
        return
    
    try:
        # Initialize client with debug mode
        print("🔧 Initializing SemanticPen client...")
        client = SemanticPenClient.create(api_key, debug=True)
        
        # Test connection
        if not client.test_connection():
            print("❌ Connection failed!")
            return
        print("✅ Connected successfully!")
        
        # Run advanced demonstrations
        demonstrate_request_builder(client)
        demonstrate_custom_polling(client)
        demonstrate_bulk_workflows(client)
        demonstrate_advanced_monitoring(client)
        demonstrate_configuration_patterns(client)
        
        print("\n🎉 Advanced usage demonstration completed!")
        print("💡 Check the generated articles in your SemanticPen dashboard")
        
    except Exception as error:
        print(f"\n❌ Error: {error}")
        sys.exit(1)


if __name__ == "__main__":
    main()