#!/usr/bin/env python3
"""
Basic usage example for SemanticPen Python SDK
Demonstrates simple article generation and retrieval
"""

import os
import sys
from semanticpen import SemanticPenClient


def main():
    """Basic usage example"""
    print("🚀 SemanticPen Python SDK - Basic Usage Example")
    print("=" * 50)
    
    # Get API key from environment variable or use placeholder
    api_key = os.getenv("SEMANTICPEN_API_KEY", "your-api-key-here")
    
    if api_key == "your-api-key-here":
        print("⚠️  Please set SEMANTICPEN_API_KEY environment variable")
        print("   Example: export SEMANTICPEN_API_KEY='your-actual-api-key'")
        return
    
    try:
        # Initialize client
        print("🔧 Initializing SemanticPen client...")
        client = SemanticPenClient.create(api_key)
        
        # Test connection
        print("🔌 Testing API connection...")
        if client.test_connection():
            print("✅ Connected successfully!")
        else:
            print("❌ Connection failed!")
            return
        
        # Generate article
        print("\n📝 Generating article...")
        keyword = "Python programming best practices"
        print(f"   Keyword: '{keyword}'")
        
        result = client.generate_article(keyword)
        
        print("✅ Article generation started!")
        print(f"   📄 Article ID: {result['articleId']}")
        print(f"   📁 Project ID: {result['projectId']}")
        print(f"   💬 Message: {result['message']}")
        
        article_id = result['articleId']
        
        # Check status
        print(f"\n🔍 Checking article status...")
        status = client.get_article_status(article_id)
        
        print(f"   📊 Status: {status['status']}")
        print(f"   🔄 Is Complete: {status['isComplete']}")
        print(f"   ❌ Has Error: {status['hasError']}")
        
        if status['errorMessage']:
            print(f"   💬 Error: {status['errorMessage']}")
        
        # Quick completion check
        print(f"\n⚡ Quick completion check...")
        is_complete = client.is_article_complete(article_id)
        print(f"   🎯 Article complete: {is_complete}")
        
        if is_complete:
            print(f"\n📖 Retrieving complete article...")
            try:
                article = client.get_article(article_id)
                print(f"   📰 Title: {article.title or 'No title yet'}")
                print(f"   📅 Created: {article.created_at}")
                print(f"   📊 Status: {article.status}")
                
                if article.article_html:
                    content_preview = article.article_html[:200] + "..." if len(article.article_html) > 200 else article.article_html
                    print(f"   📝 Content preview: {content_preview}")
                else:
                    print(f"   📝 Content: Still generating...")
                    
            except Exception as e:
                print(f"   ⏳ Article still processing: {e}")
        else:
            print(f"   ⏳ Article is still being generated...")
            print(f"   💡 You can monitor this article in your SemanticPen dashboard")
        
        print(f"\n🎉 Basic usage example completed!")
        print(f"📝 Generated article ID for monitoring: {article_id}")
        
    except Exception as error:
        print(f"\n❌ Error: {error}")
        sys.exit(1)


if __name__ == "__main__":
    main()