#!/usr/bin/env python3
"""
Bulk generation example for SemanticPen Python SDK
Demonstrates generating multiple articles simultaneously
"""

import os
import sys
import time
from semanticpen import SemanticPenClient, PollingConfig


def main():
    """Bulk generation example"""
    print("🚀 SemanticPen Python SDK - Bulk Generation Example")
    print("=" * 55)
    
    # Get API key from environment variable
    api_key = os.getenv("SEMANTICPEN_API_KEY", "your-api-key-here")
    
    if api_key == "your-api-key-here":
        print("⚠️  Please set SEMANTICPEN_API_KEY environment variable")
        return
    
    try:
        # Initialize client
        print("🔧 Initializing SemanticPen client...")
        client = SemanticPenClient.create(api_key, debug=True)
        
        # Test connection
        if not client.test_connection():
            print("❌ Connection failed!")
            return
        print("✅ Connected successfully!")
        
        # Define keywords for bulk generation
        keywords = [
            "Python web development frameworks",
            "Machine learning algorithms explained",
            "RESTful API design principles",
            "Database optimization techniques",
            "Cloud computing best practices"
        ]
        
        print(f"\n📝 Generating {len(keywords)} articles...")
        for i, keyword in enumerate(keywords, 1):
            print(f"   {i}. {keyword}")
        
        # Generate articles in bulk
        start_time = time.time()
        results = client.generate_articles(keywords)
        generation_time = time.time() - start_time
        
        print(f"\n✅ Bulk generation completed in {generation_time:.2f} seconds")
        print(f"   ✓ Successful: {results.success_count}")
        print(f"   ✗ Failed: {results.failure_count}")
        print(f"   📊 Total: {results.total}")
        
        # Show successful generations
        if results.successful:
            print(f"\n🎉 Successfully generated articles:")
            for item in results.successful:
                print(f"   📄 {item['keyword']} → {item['article_id']}")
        
        # Show failures
        if results.failed:
            print(f"\n❌ Failed generations:")
            for item in results.failed:
                print(f"   ✗ {item['item']}: {item['error']}")
        
        # If we have successful articles, demonstrate bulk status checking
        if results.successful:
            article_ids = [item['article_id'] for item in results.successful]
            
            print(f"\n🔍 Checking status of all articles...")
            status_results = client.get_multiple_status(article_ids)
            
            print(f"   ✅ Completed: {len(status_results['completed'])}")
            print(f"   ⏳ Pending: {len(status_results['pending'])}")
            print(f"   ❌ Failed: {len(status_results['failed'])}")
            print(f"   🔍 Not Found: {len(status_results['notFound'])}")
            
            # Demonstrate waiting for completion with progress tracking
            if status_results['pending']:
                print(f"\n⏳ Waiting for articles to complete...")
                
                def progress_callback(completed, total, new_articles):
                    print(f"   📊 Progress: {completed}/{total} articles completed")
                    for article in new_articles:
                        print(f"      ✓ {article.title or 'Untitled'}")
                
                # Custom polling configuration
                polling_config = PollingConfig(
                    interval=10000,  # Check every 10 seconds
                    max_attempts=30,  # Max 5 minutes
                    backoff_multiplier=1.1
                )
                
                completion_start = time.time()
                completion_results = client.wait_for_articles(
                    article_ids,
                    polling_config=polling_config,
                    on_progress=progress_callback
                )
                completion_time = time.time() - completion_start
                
                print(f"\n🎉 Waiting completed in {completion_time:.2f} seconds")
                print(f"   ✅ Completed: {len(completion_results['completed'])}")
                print(f"   ❌ Failed: {len(completion_results['failed'])}")
                
                # Show completed articles
                for article in completion_results['completed']:
                    print(f"   📰 {article.title} (ID: {article.id})")
        
        # Demonstrate complete bulk workflow (generate + wait)
        print(f"\n🔄 Demonstrating complete bulk workflow...")
        
        workflow_keywords = [
            "Docker containerization guide",
            "Microservices architecture patterns"
        ]
        
        def workflow_progress(completed, total, new_articles):
            print(f"   🔄 Workflow progress: {completed}/{total}")
        
        workflow_start = time.time()
        workflow_result = client.generate_bulk_and_wait(
            workflow_keywords,
            generation_options={"project_name": "SDK Example Articles"},
            on_progress=workflow_progress
        )
        workflow_time = time.time() - workflow_start
        
        print(f"\n✅ Complete workflow finished in {workflow_time:.2f} seconds")
        print(f"   ✓ Successful: {len(workflow_result['successful'])}")
        print(f"   ✗ Failed: {len(workflow_result['failed'])}")
        
        # Show final results
        for article in workflow_result['successful']:
            print(f"   📰 {article.title} (Length: {len(article.article_html or '')} chars)")
        
        print(f"\n🎉 Bulk generation example completed!")
        
    except Exception as error:
        print(f"\n❌ Error: {error}")
        sys.exit(1)


if __name__ == "__main__":
    main()