#!/usr/bin/env python3
"""
Error handling example for SemanticPen Python SDK
Demonstrates comprehensive error handling patterns
"""

import os
import sys
from semanticpen import (
    SemanticPenClient,
    SemanticPenConfig,
    AuthenticationError,
    ValidationError,
    RateLimitError,
    NetworkError,
    TimeoutError,
    ApiError,
    UnknownError
)


def test_authentication_error():
    """Test authentication error handling"""
    print("🔐 Testing authentication error...")
    
    try:
        # Use invalid API key
        client = SemanticPenClient.create("invalid-api-key")
        client.test_connection()
        print("   ❌ Expected authentication error but didn't get one")
    except AuthenticationError as e:
        print(f"   ✅ Caught authentication error: {e}")
        print(f"      Type: {e.type}")
        print(f"      Timestamp: {e.timestamp}")
    except Exception as e:
        print(f"   ⚠️  Got different error: {type(e).__name__}: {e}")


def test_validation_errors(client):
    """Test validation error handling"""
    print("\n📝 Testing validation errors...")
    
    # Test empty keyword
    try:
        client.generate_article("")
        print("   ❌ Expected validation error for empty keyword")
    except ValidationError as e:
        print(f"   ✅ Caught validation error: {e}")
        if hasattr(e, 'field'):
            print(f"      Field: {e.field}")
        if hasattr(e, 'value'):
            print(f"      Value: {e.value}")
    
    # Test invalid keyword type
    try:
        client.generate_article(None)
        print("   ❌ Expected validation error for None keyword")
    except (ValidationError, TypeError) as e:
        print(f"   ✅ Caught validation error: {e}")
    
    # Test empty keywords array
    try:
        client.generate_articles([])
        print("   ❌ Expected validation error for empty array")
    except ValidationError as e:
        print(f"   ✅ Caught validation error: {e}")
    
    # Test invalid keywords in array
    try:
        client.generate_articles(["valid", "", "also valid"])
        print("   ❌ Expected validation error for invalid keyword in array")
    except ValidationError as e:
        print(f"   ✅ Caught validation error: {e}")


def test_rate_limit_handling(client):
    """Test rate limit error handling"""
    print("\n🚦 Testing rate limit handling...")
    
    # Note: This is a simulation - actual rate limits depend on your plan
    try:
        # Rapid-fire requests to potentially trigger rate limits
        for i in range(5):
            result = client.generate_article(f"Test article {i}")
            print(f"   Request {i+1}: {result['articleId']}")
    except RateLimitError as e:
        print(f"   ✅ Caught rate limit error: {e}")
        if hasattr(e, 'retry_after'):
            print(f"      Retry after: {e.retry_after} seconds")
        if hasattr(e, 'daily_limit'):
            print(f"      Daily limit: {e.daily_limit}")
        if hasattr(e, 'current_usage'):
            print(f"      Current usage: {e.current_usage}")
    except Exception as e:
        print(f"   ℹ️  No rate limit hit: {type(e).__name__}: {e}")


def test_network_errors():
    """Test network error handling"""
    print("\n🌐 Testing network errors...")
    
    try:
        # Use invalid base URL to trigger network error
        config = SemanticPenConfig(
            api_key="test-key",
            base_url="https://invalid-domain-that-does-not-exist.com",
            timeout=5000
        )
        client = SemanticPenClient.create_with_config(config)
        client.test_connection()
        print("   ❌ Expected network error but didn't get one")
    except NetworkError as e:
        print(f"   ✅ Caught network error: {e}")
        if hasattr(e, 'status_code'):
            print(f"      Status code: {e.status_code}")
        if hasattr(e, 'endpoint'):
            print(f"      Endpoint: {e.endpoint}")
    except Exception as e:
        print(f"   ✅ Caught network-related error: {type(e).__name__}: {e}")


def test_timeout_errors():
    """Test timeout error handling"""
    print("\n⏰ Testing timeout errors...")
    
    try:
        # Use very short timeout to trigger timeout error
        config = SemanticPenConfig(
            api_key=os.getenv("SEMANTICPEN_API_KEY", "test-key"),
            timeout=1  # 1ms - guaranteed to timeout
        )
        client = SemanticPenClient.create_with_config(config)
        client.test_connection()
        print("   ❌ Expected timeout error but didn't get one")
    except TimeoutError as e:
        print(f"   ✅ Caught timeout error: {e}")
        if hasattr(e, 'timeout_ms'):
            print(f"      Timeout: {e.timeout_ms}ms")
        if hasattr(e, 'operation'):
            print(f"      Operation: {e.operation}")
    except Exception as e:
        print(f"   ✅ Caught timeout-related error: {type(e).__name__}: {e}")


def test_api_errors(client):
    """Test API-specific error handling"""
    print("\n🔌 Testing API errors...")
    
    try:
        # Try to get non-existent article
        client.get_article("non-existent-article-id")
        print("   ❌ Expected API error for non-existent article")
    except ApiError as e:
        print(f"   ✅ Caught API error: {e}")
        if hasattr(e, 'status_code'):
            print(f"      Status code: {e.status_code}")
        if hasattr(e, 'endpoint'):
            print(f"      Endpoint: {e.endpoint}")
    except Exception as e:
        print(f"   ✅ Caught API-related error: {type(e).__name__}: {e}")


def test_error_serialization():
    """Test error serialization and logging"""
    print("\n📄 Testing error serialization...")
    
    try:
        raise ValidationError(
            "Test validation error",
            field="test_field",
            value="test_value",
            details={"extra": "info"}
        )
    except ValidationError as e:
        print("   ✅ Error details:")
        print(f"      JSON: {e.to_json()}")
        print(f"      Dict: {e.to_dict()}")
        print(f"      Formatted: {e.format()}")


def demonstrate_error_recovery(client):
    """Demonstrate error recovery patterns"""
    print("\n🔄 Demonstrating error recovery patterns...")
    
    # Retry pattern
    max_retries = 3
    retry_count = 0
    
    while retry_count < max_retries:
        try:
            # This might fail due to various reasons
            result = client.generate_article("Error recovery test")
            print(f"   ✅ Success on attempt {retry_count + 1}")
            print(f"      Article ID: {result['articleId']}")
            break
        except RateLimitError as e:
            retry_count += 1
            if retry_count >= max_retries:
                print(f"   ❌ Max retries reached for rate limit")
                break
            
            wait_time = getattr(e, 'retry_after', 5)
            print(f"   ⏳ Rate limited, waiting {wait_time}s (attempt {retry_count})")
            # time.sleep(wait_time)  # Uncomment for actual retry
        except ValidationError as e:
            print(f"   ❌ Validation error (no retry): {e}")
            break
        except NetworkError as e:
            retry_count += 1
            if retry_count >= max_retries:
                print(f"   ❌ Max retries reached for network error")
                break
            print(f"   🌐 Network error, retrying (attempt {retry_count})")
        except Exception as e:
            print(f"   ❌ Unexpected error: {type(e).__name__}: {e}")
            break


def main():
    """Main error handling demonstration"""
    print("🚀 SemanticPen Python SDK - Error Handling Example")
    print("=" * 55)
    
    # Test authentication errors
    test_authentication_error()
    
    # Test network errors
    test_network_errors()
    
    # Test timeout errors
    test_timeout_errors()
    
    # Test error serialization
    test_error_serialization()
    
    # Get API key for remaining tests
    api_key = os.getenv("SEMANTICPEN_API_KEY")
    
    if not api_key:
        print("\n⚠️  Set SEMANTICPEN_API_KEY for additional error tests")
        return
    
    try:
        # Initialize client for other tests
        client = SemanticPenClient.create(api_key)
        
        # Test validation errors
        test_validation_errors(client)
        
        # Test API errors
        test_api_errors(client)
        
        # Test rate limit handling (if applicable)
        test_rate_limit_handling(client)
        
        # Demonstrate error recovery
        demonstrate_error_recovery(client)
        
        print("\n🎉 Error handling demonstration completed!")
        
    except Exception as error:
        print(f"\n❌ Setup error: {error}")


if __name__ == "__main__":
    main()