"""
SemanticPen Python SDK
AI Article Writer & SEO Blog Generator SDK

A professional Python library for automated content creation, AI-powered article 
generation, and SEO blog writing with SemanticPen.

Example:
    Basic usage:
    
    >>> from semanticpen import SemanticPenClient
    >>> client = SemanticPenClient.create("your-api-key")
    >>> result = client.generate_article("Python programming best practices")
    >>> print(f"Article ID: {result['articleId']}")
    
    Advanced usage with configuration:
    
    >>> from semanticpen import SemanticPenClient, SemanticPenConfig
    >>> config = SemanticPenConfig(
    ...     api_key="your-api-key",
    ...     base_url="https://www.semanticpen.com",
    ...     timeout=60000,
    ...     debug=True
    ... )
    >>> client = SemanticPenClient.create_with_config(config)
    >>> 
    >>> # Generate and wait for completion
    >>> article = client.generate_and_wait("AI writing tools")
    >>> print(f"Title: {article.title}")
    >>> print(f"Content: {article.article_html[:100]}...")
"""

from .core import SemanticPenClient
from .types import (
    SemanticPenConfig,
    AuthResult,
    ArticleGenerationRequest,
    ArticleGenerationResponse,
    Article,
    ArticleStatusInfo,
    ArticleStatusEnum,
    ApiError,
    ErrorType,
    RequestOptions,
    PollingConfig,
    BulkOperationResult,
)
from .errors import (
    SemanticPenError,
    AuthenticationError,
    ValidationError,
    RateLimitError,
    NetworkError,
    ApiError as ApiErrorException,
    TimeoutError,
    UnknownError,
    ErrorFactory,
)

# Version information
__version__ = "1.0.0"
__author__ = "SemanticPen"
__email__ = "contact@semanticpen.com"
__url__ = "https://www.semanticpen.com"
__description__ = "AI Article Writer & SEO Blog Generator SDK"

# Public API
__all__ = [
    # Core client
    "SemanticPenClient",
    
    # Configuration and Authentication
    "SemanticPenConfig",
    "AuthResult",
    
    # Article Generation
    "ArticleGenerationRequest",
    "ArticleGenerationResponse", 
    
    # Article Retrieval
    "Article",
    "ArticleStatusInfo",
    "ArticleStatusEnum",
    
    # Error Handling
    "SemanticPenError",
    "AuthenticationError",
    "ValidationError", 
    "RateLimitError",
    "NetworkError",
    "ApiErrorException",
    "TimeoutError",
    "UnknownError",
    "ErrorFactory",
    "ApiError",
    "ErrorType",
    
    # Utilities
    "RequestOptions",
    "PollingConfig",
    "BulkOperationResult",
    
    # Package metadata
    "__version__",
    "__author__",
    "__email__",
    "__url__",
    "__description__",
]