"""
Base client class implementing core HTTP functionality
Abstract base class using inheritance and composition patterns
"""

import json
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, Union

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from ..types.models import SemanticPenConfig, RequestOptions
from ..errors import ErrorFactory, NetworkError, TimeoutError, ValidationError


class BaseClient(ABC):
    """
    Abstract base class for all API clients
    Implements common HTTP functionality and error handling
    """
    
    def __init__(self, config: SemanticPenConfig):
        self.config = config
        self.base_url = config.base_url.rstrip("/")
        self.timeout = config.timeout / 1000.0  # Convert ms to seconds
        self.debug = config.debug
        
        self._validate_config()
        self._setup_session()
    
    def _validate_config(self) -> None:
        """Validate client configuration"""
        if not self.config.api_key or not isinstance(self.config.api_key, str):
            raise ValidationError("API key is required and must be a string")
        
        if len(self.config.api_key) < 10:
            raise ValidationError("API key appears to be invalid (too short)")
        
        if self.timeout < 1 or self.timeout > 300:
            raise ValidationError("Timeout must be between 1000ms and 300000ms")
    
    def _setup_session(self) -> None:
        """Setup requests session with retry strategy"""
        self.session = requests.Session()
        
        # Configure retry strategy
        retry_strategy = Retry(
            total=3,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=["HEAD", "GET", "OPTIONS"],
            backoff_factor=1
        )
        
        adapter = HTTPAdapter(max_retries=retry_strategy)
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)
    
    def _get_default_headers(self) -> Dict[str, str]:
        """Get default headers for all requests"""
        return {
            "Authorization": f"Bearer {self.config.api_key}",
            "Content-Type": "application/json",
            "Accept": "application/json", 
            "User-Agent": "SemanticPen-Python-SDK/1.0.0"
        }
    
    def _build_url(self, endpoint: str) -> str:
        """Build full URL from endpoint"""
        clean_endpoint = endpoint.lstrip("/")
        return f"{self.base_url}/{clean_endpoint}"
    
    def _log_request(self, method: str, url: str, body: Any = None) -> None:
        """Log request details if debug is enabled"""
        if self.debug:
            print(f"[SemanticPen SDK] {method} {url}")
            if body:
                print(f"[SemanticPen SDK] Request body: {body}")
    
    def _log_response(self, response: requests.Response, response_data: Any = None) -> None:
        """Log response details if debug is enabled"""
        if self.debug:
            print(f"[SemanticPen SDK] Response status: {response.status_code}")
            if response_data:
                print(f"[SemanticPen SDK] Response data: {response_data}")
    
    def request(
        self,
        endpoint: str,
        method: str = "GET",
        body: Optional[Any] = None,
        headers: Optional[Dict[str, str]] = None,
        timeout: Optional[float] = None
    ) -> Any:
        """
        Core HTTP request method
        Implements timeout, error handling, and response parsing
        """
        url = self._build_url(endpoint)
        request_timeout = timeout or self.timeout
        
        self._log_request(method, url, body)
        
        # Prepare headers
        request_headers = self._get_default_headers()
        if headers:
            request_headers.update(headers)
        
        # Prepare request kwargs
        kwargs = {
            "method": method,
            "url": url,
            "headers": request_headers,
            "timeout": request_timeout
        }
        
        # Add body for non-GET requests
        if body is not None and method != "GET":
            if isinstance(body, (dict, list)):
                kwargs["json"] = body
            else:
                kwargs["data"] = body
        
        try:
            response = self.session.request(**kwargs)
            
            # Parse response data
            response_data = None
            content_type = response.headers.get("content-type", "")
            
            if "application/json" in content_type:
                try:
                    response_data = response.json()
                except json.JSONDecodeError:
                    response_data = None
            else:
                response_data = response.text
            
            self._log_response(response, response_data)
            
            # Handle non-2xx responses
            if not response.ok:
                raise ErrorFactory.from_http_response(response, endpoint, response_data)
            
            return response_data
            
        except requests.exceptions.Timeout:
            raise TimeoutError(
                f"Request timed out after {request_timeout}s",
                int(request_timeout * 1000),
                f"{method} {endpoint}"
            )
        
        except requests.exceptions.ConnectionError as e:
            raise NetworkError(
                "Network error: Unable to reach SemanticPen API",
                endpoint=endpoint,
                details={"original_error": str(e)}
            )
        
        except requests.exceptions.RequestException as e:
            # Re-raise SemanticPen errors
            if hasattr(e, 'type'):
                raise e
            
            # Convert other errors
            raise ErrorFactory.from_exception(e, f"{method} {endpoint}")
    
    def get(self, endpoint: str, headers: Optional[Dict[str, str]] = None) -> Any:
        """GET request helper"""
        return self.request(endpoint, "GET", headers=headers)
    
    def post(
        self,
        endpoint: str,
        body: Optional[Any] = None,
        headers: Optional[Dict[str, str]] = None
    ) -> Any:
        """POST request helper"""
        return self.request(endpoint, "POST", body=body, headers=headers)
    
    def put(
        self,
        endpoint: str,
        body: Optional[Any] = None,
        headers: Optional[Dict[str, str]] = None
    ) -> Any:
        """PUT request helper"""
        return self.request(endpoint, "PUT", body=body, headers=headers)
    
    def delete(self, endpoint: str, headers: Optional[Dict[str, str]] = None) -> Any:
        """DELETE request helper"""
        return self.request(endpoint, "DELETE", headers=headers)
    
    def get_config(self) -> SemanticPenConfig:
        """Get current configuration (copy)"""
        return SemanticPenConfig(
            api_key=self.config.api_key,
            base_url=self.config.base_url,
            timeout=self.config.timeout,
            debug=self.config.debug
        )
    
    @abstractmethod
    def test_connection(self) -> bool:
        """Test API connection"""
        pass