"""
Main SemanticPen SDK client class
Professional-grade client implementing composition and facade patterns
"""

import time
from typing import Any, Dict, List, Optional, Union

from .base_client import BaseClient
from ..services.article_service import ArticleService
from ..utils.status_poller import StatusPoller
from ..types.models import (
    SemanticPenConfig,
    ArticleGenerationRequest,
    ArticleGenerationResponse,
    Article,
    PollingConfig,
    BulkOperationResult,
)
from ..errors import ValidationError


class SemanticPenClient:
    """
    Main client class for SemanticPen API
    Implements facade pattern to provide simplified interface
    """
    
    def __init__(self, config: SemanticPenConfig):
        self.config = config
        self.article_service = ArticleService(config)
        self.status_poller = StatusPoller(self.article_service)
    
    # ============================================================================
    # STATIC FACTORY METHODS
    # ============================================================================
    
    @classmethod
    def create(
        cls,
        api_key: str,
        base_url: str = "https://www.semanticpen.com",
        timeout: int = 30000,
        debug: bool = False
    ) -> "SemanticPenClient":
        """
        Create client instance with API key
        Factory method for easier instantiation
        """
        config = SemanticPenConfig(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            debug=debug
        )
        return cls(config)
    
    @classmethod
    def create_with_config(cls, config: SemanticPenConfig) -> "SemanticPenClient":
        """Create client with full configuration"""
        return cls(config)
    
    # ============================================================================
    # CONNECTION AND AUTHENTICATION
    # ============================================================================
    
    def test_connection(self) -> bool:
        """
        Test connection and authentication
        Validates API key and service availability
        """
        return self.article_service.test_connection()
    
    def get_config(self) -> SemanticPenConfig:
        """Get current client configuration"""
        return self.article_service.get_config()
    
    # ============================================================================
    # ARTICLE GENERATION METHODS
    # ============================================================================
    
    def generate_article(
        self,
        keyword: str,
        **options
    ) -> Dict[str, str]:
        """
        Generate a single article from keyword
        Most common use case - simplified interface
        """
        return self.article_service.generate_article(keyword, options)
    
    def generate_articles(
        self,
        keywords: List[str],
        **options
    ) -> BulkOperationResult:
        """
        Generate multiple articles from keywords
        Handles bulk generation with shared configuration
        """
        return self.article_service.generate_bulk_articles(keywords, options)
    
    def generate_with_request(self, request: ArticleGenerationRequest) -> ArticleGenerationResponse:
        """
        Generate articles with custom request
        Full control over generation parameters
        """
        return self.article_service.generate_articles(request)
    
    # ============================================================================
    # ARTICLE RETRIEVAL METHODS
    # ============================================================================
    
    def get_article(self, article_id: str) -> Article:
        """
        Get article by ID
        Returns complete article with content and metadata
        """
        return self.article_service.get_article(article_id)
    
    def get_articles(self, article_ids: List[str]) -> BulkOperationResult:
        """
        Get multiple articles by IDs
        Handles bulk retrieval with error handling
        """
        return self.article_service.get_articles(article_ids)
    
    def is_article_complete(self, article_id: str) -> bool:
        """
        Check if article is complete
        Quick status check without full article data
        """
        return self.article_service.is_article_complete(article_id)
    
    def get_article_status(self, article_id: str) -> Dict[str, Any]:
        """
        Get article status information
        Returns status without full content
        """
        return self.article_service.get_article_status(article_id)
    
    # ============================================================================
    # POLLING AND WAITING METHODS
    # ============================================================================
    
    def wait_for_article(
        self,
        article_id: str,
        polling_config: Optional[PollingConfig] = None,
        callbacks: Optional[Dict[str, Any]] = None
    ) -> Article:
        """
        Wait for article generation to complete
        Polls until article is ready or timeout
        """
        result = self.status_poller.poll_until_complete(
            article_id,
            polling_config,
            callbacks
        )
        
        if not result.get("success") or not result.get("article"):
            error_msg = result.get("error", "Failed to wait for article completion")
            raise Exception(error_msg)
        
        return result["article"]
    
    def wait_for_articles(
        self,
        article_ids: List[str],
        polling_config: Optional[PollingConfig] = None,
        on_progress: Optional[callable] = None
    ) -> Dict[str, Any]:
        """
        Wait for multiple articles to complete
        Handles bulk waiting with progress tracking
        """
        return self.status_poller.wait_for_generation_complete(
            article_ids,
            polling_config,
            on_progress
        )
    
    def get_multiple_status(self, article_ids: List[str]) -> Dict[str, List[Any]]:
        """
        Get status of multiple articles without polling
        One-time status check for bulk articles
        """
        return self.status_poller.get_multiple_status(article_ids)
    
    # ============================================================================
    # CONVENIENCE WORKFLOWS
    # ============================================================================
    
    def generate_and_wait(
        self,
        keyword: str,
        generation_options: Optional[Dict[str, Any]] = None,
        polling_config: Optional[PollingConfig] = None,
        callbacks: Optional[Dict[str, Any]] = None
    ) -> Article:
        """
        Generate article and wait for completion
        Complete workflow in one method call
        """
        # Generate article
        generation = self.generate_article(keyword, **(generation_options or {}))
        
        # Wait for completion
        return self.wait_for_article(
            generation["articleId"],
            polling_config,
            callbacks
        )
    
    def generate_bulk_and_wait(
        self,
        keywords: List[str],
        generation_options: Optional[Dict[str, Any]] = None,
        polling_config: Optional[PollingConfig] = None,
        on_progress: Optional[callable] = None
    ) -> Dict[str, Any]:
        """
        Generate multiple articles and wait for all to complete
        Complete bulk workflow with progress tracking
        """
        start_time = time.time()
        
        # Generate articles
        generation = self.generate_articles(keywords, **(generation_options or {}))
        
        if generation.success_count == 0:
            return {
                "successful": [],
                "failed": [{"keyword": f.get("item"), "error": f.get("error")} for f in generation.failed],
                "total_time": time.time() - start_time
            }
        
        # Extract article IDs
        article_ids = [s.get("article_id") for s in generation.successful]
        
        # Wait for completion
        completion = self.wait_for_articles(
            article_ids,
            polling_config,
            on_progress
        )
        
        # Combine generation and completion failures
        all_failed = []
        all_failed.extend([{"keyword": f.get("item"), "error": f.get("error")} for f in generation.failed])
        
        for f in completion.get("failed", []):
            success_item = next(
                (s for s in generation.successful if s.get("article_id") == f.get("article_id")),
                None
            )
            keyword = success_item.get("keyword") if success_item else f.get("article_id")
            all_failed.append({"keyword": keyword, "error": f.get("error")})
        
        return {
            "successful": completion.get("completed", []),
            "failed": all_failed,
            "total_time": time.time() - start_time
        }
    
    # ============================================================================
    # UTILITY METHODS
    # ============================================================================
    
    @staticmethod
    def validate_keyword(keyword: str) -> bool:
        """
        Validate keyword format
        Helper method for input validation
        """
        return isinstance(keyword, str) and len(keyword.strip()) > 0
    
    @staticmethod
    def validate_keywords(keywords: List[str]) -> Dict[str, Any]:
        """
        Validate multiple keywords
        Helper method for bulk operations
        """
        errors = []
        
        if not isinstance(keywords, list):
            errors.append("Keywords must be a list")
            return {"valid": False, "errors": errors}
        
        if len(keywords) == 0:
            errors.append("Keywords list cannot be empty")
            return {"valid": False, "errors": errors}
        
        for index, keyword in enumerate(keywords):
            if not SemanticPenClient.validate_keyword(keyword):
                errors.append(f"Invalid keyword at index {index}: {keyword}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors
        }
    
    def create_request_builder(self) -> "ArticleRequestBuilder":
        """
        Create request builder for complex configurations
        Builder pattern for advanced use cases
        """
        return ArticleRequestBuilder(self)


# ============================================================================
# REQUEST BUILDER CLASS
# ============================================================================

class ArticleRequestBuilder:
    """
    Builder class for constructing complex article generation requests
    Implements builder pattern for fluent API
    """
    
    def __init__(self, client: SemanticPenClient):
        self.client = client
        self.request = {}
    
    def keyword(self, keyword: Union[str, List[str]]) -> "ArticleRequestBuilder":
        """Set target keyword(s)"""
        self.request["target_keyword"] = keyword
        return self
    
    def gpt_version(self, version: str) -> "ArticleRequestBuilder":
        """Set GPT version"""
        self.request["gpt_version"] = version
        return self
    
    def integration(self, integration_id: str) -> "ArticleRequestBuilder":
        """Set integration ID"""
        self.request["integration_id"] = integration_id
        return self
    
    def preset(self, preset_id: str) -> "ArticleRequestBuilder":
        """Set preset ID"""
        self.request["preset_id"] = preset_id
        return self
    
    def project_name(self, name: str) -> "ArticleRequestBuilder":
        """Set project name"""
        self.request["project_name"] = name
        return self
    
    def personalization(self, name: str) -> "ArticleRequestBuilder":
        """Set personalization"""
        self.request["personalisation_name"] = name
        return self
    
    def parameter(self, key: str, value: Any) -> "ArticleRequestBuilder":
        """Add custom parameter"""
        if not self.request.get("additional_params"):
            self.request["additional_params"] = {}
        self.request["additional_params"][key] = value
        return self
    
    def generate(self) -> ArticleGenerationResponse:
        """Build and execute the request"""
        if not self.request.get("target_keyword"):
            raise ValidationError("Target keyword is required")
        
        request_obj = ArticleGenerationRequest(**self.request)
        return self.client.generate_with_request(request_obj)
    
    def generate_and_wait(
        self,
        polling_config: Optional[PollingConfig] = None,
        callbacks: Optional[Dict[str, Any]] = None
    ) -> Union[Article, List[Article]]:
        """Build, execute, and wait for completion"""
        response = self.generate()
        
        if response.article_ids and len(response.article_ids) == 1:
            return self.client.wait_for_article(
                response.article_ids[0],
                polling_config,
                callbacks
            )
        elif response.article_ids:
            result = self.client.wait_for_articles(
                response.article_ids,
                polling_config
            )
            return result.get("completed", [])
        else:
            raise ValidationError("No article IDs returned from generation")
    
    def get_request(self) -> Dict[str, Any]:
        """Get current request configuration"""
        return dict(self.request)