"""
Professional error handling classes for SemanticPen SDK
"""

import json
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Any, Dict, Optional, Union

from ..types.enums import ErrorType


# ============================================================================
# BASE ERROR CLASS
# ============================================================================

class SemanticPenError(Exception, ABC):
    """
    Base abstract error class for all SemanticPen SDK errors
    Implements proper inheritance and error handling patterns
    """
    
    def __init__(
        self,
        message: str,
        error_type: ErrorType,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message)
        self.message = message
        self.type = error_type
        self.timestamp = datetime.now()
        self.details = details or {}
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert error to dictionary for logging/debugging"""
        return {
            "name": self.__class__.__name__,
            "message": self.message,
            "type": self.type.value,
            "timestamp": self.timestamp.isoformat(),
            "details": self.details,
        }
    
    def to_json(self) -> str:
        """Convert error to JSON string"""
        return json.dumps(self.to_dict(), indent=2)
    
    @abstractmethod
    def format(self) -> str:
        """Abstract method for error-specific formatting"""
        pass
    
    def __str__(self) -> str:
        return self.format()


# ============================================================================
# SPECIFIC ERROR CLASSES
# ============================================================================

class AuthenticationError(SemanticPenError):
    """Authentication and authorization errors"""
    
    def __init__(
        self,
        message: str = "Authentication failed",
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.AUTHENTICATION_ERROR, details)
    
    def format(self) -> str:
        return f"Authentication Error: {self.message}"


class ValidationError(SemanticPenError):
    """Input validation errors"""
    
    def __init__(
        self,
        message: str,
        field: Optional[str] = None,
        value: Any = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.VALIDATION_ERROR, details)
        self.field = field
        self.value = value
    
    def format(self) -> str:
        if self.field:
            return f"Validation Error in field '{self.field}': {self.message}"
        return f"Validation Error: {self.message}"


class RateLimitError(SemanticPenError):
    """Rate limiting errors"""
    
    def __init__(
        self,
        message: str = "Rate limit exceeded",
        retry_after: Optional[int] = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.RATE_LIMIT_ERROR, details)
        self.retry_after = retry_after
        self.daily_limit = details.get("daily_limit") if details else None
        self.current_usage = details.get("current_usage") if details else None
    
    def format(self) -> str:
        formatted = f"Rate Limit Error: {self.message}"
        if self.retry_after:
            formatted += f" (Retry after {self.retry_after} seconds)"
        if self.daily_limit and self.current_usage:
            formatted += f" (Usage: {self.current_usage}/{self.daily_limit})"
        return formatted


class NetworkError(SemanticPenError):
    """Network-related errors"""
    
    def __init__(
        self,
        message: str,
        status_code: Optional[int] = None,
        endpoint: Optional[str] = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.NETWORK_ERROR, details)
        self.status_code = status_code
        self.endpoint = endpoint
    
    def format(self) -> str:
        formatted = f"Network Error: {self.message}"
        if self.status_code:
            formatted += f" (Status: {self.status_code})"
        if self.endpoint:
            formatted += f" (Endpoint: {self.endpoint})"
        return formatted


class ApiError(SemanticPenError):
    """API-specific errors returned by SemanticPen service"""
    
    def __init__(
        self,
        message: str,
        status_code: int,
        endpoint: str,
        response: Optional[Any] = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.API_ERROR, details)
        self.status_code = status_code
        self.endpoint = endpoint
        self.response = response
    
    def format(self) -> str:
        return f"API Error [{self.status_code}] at {self.endpoint}: {self.message}"
    
    @classmethod
    def from_response(
        cls,
        response,  # requests.Response
        endpoint: str,
        response_data: Optional[Any] = None
    ) -> "ApiError":
        """Create ApiError from HTTP response"""
        message = "Unknown API error"
        if response_data and isinstance(response_data, dict):
            message = response_data.get("error", f"HTTP {response.status_code}: {response.reason}")
        elif hasattr(response, 'text'):
            message = response.text or f"HTTP {response.status_code}: {response.reason}"
        
        return cls(message, response.status_code, endpoint, response_data)


class TimeoutError(SemanticPenError):
    """Timeout errors for long-running operations"""
    
    def __init__(
        self,
        message: str,
        timeout_ms: int,
        operation: Optional[str] = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.TIMEOUT_ERROR, details)
        self.timeout_ms = timeout_ms
        self.operation = operation
    
    def format(self) -> str:
        formatted = f"Timeout Error: {self.message} ({self.timeout_ms}ms)"
        if self.operation:
            formatted += f" during {self.operation}"
        return formatted


class UnknownError(SemanticPenError):
    """Generic unknown errors"""
    
    def __init__(
        self,
        message: str = "An unknown error occurred",
        original_error: Optional[Exception] = None,
        details: Optional[Dict[str, Any]] = None
    ):
        super().__init__(message, ErrorType.UNKNOWN_ERROR, details)
        self.original_error = original_error
    
    def format(self) -> str:
        formatted = f"Unknown Error: {self.message}"
        if self.original_error:
            formatted += f" (Original: {str(self.original_error)})"
        return formatted


# ============================================================================
# ERROR FACTORY
# ============================================================================

class ErrorFactory:
    """
    Factory class for creating appropriate error instances
    Implements factory pattern for error creation
    """
    
    @staticmethod
    def from_http_response(
        response,  # requests.Response
        endpoint: str,
        response_data: Optional[Any] = None
    ) -> SemanticPenError:
        """Create error from HTTP response"""
        status_code = response.status_code
        
        if status_code in (401, 403):
            message = "Authentication failed"
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
            return AuthenticationError(
                message,
                details={"status_code": status_code, "endpoint": endpoint}
            )
        
        elif status_code == 400:
            message = "Invalid request"
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
            return ValidationError(
                message,
                details={"status_code": status_code, "endpoint": endpoint}
            )
        
        elif status_code == 429:
            message = "Rate limit exceeded"
            retry_after = None
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
                retry_after = response_data.get("retry_after")
            return RateLimitError(
                message,
                retry_after=retry_after,
                details={"status_code": status_code, "endpoint": endpoint}
            )
        
        elif status_code == 404:
            message = "Resource not found"
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
            return ApiError(message, status_code, endpoint, response_data)
        
        elif status_code in (500, 502, 503, 504):
            message = "Server error"
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
            return NetworkError(
                message,
                status_code=status_code,
                endpoint=endpoint,
                details={"response_data": response_data}
            )
        
        else:
            message = f"HTTP {status_code}"
            if response_data and isinstance(response_data, dict):
                message = response_data.get("error", message)
            return ApiError(message, status_code, endpoint, response_data)
    
    @staticmethod
    def from_exception(
        error: Exception,
        context: Optional[str] = None
    ) -> SemanticPenError:
        """Create error from generic Python exception"""
        if isinstance(error, SemanticPenError):
            return error
        
        if isinstance(error, (TypeError, ValueError)):
            return ValidationError(
                str(error),
                details={"original_error": str(error), "context": context}
            )
        
        if isinstance(error, (ConnectionError, TimeoutError)):
            return NetworkError(
                str(error),
                details={"original_error": str(error), "context": context}
            )
        
        return UnknownError(
            str(error) or "An unknown error occurred",
            original_error=error,
            details={"context": context}
        )