"""
Article service class handling article generation and retrieval
Implements composition pattern with BaseClient
"""

import asyncio
from typing import Any, Dict, List, Optional, Union

from ..core.base_client import BaseClient
from ..types.models import (
    ArticleGenerationRequest,
    ArticleGenerationResponse,
    Article,
    BulkOperationResult,
    SemanticPenConfig,
)
from ..errors import ValidationError


class ArticleService(BaseClient):
    """
    Service class for article-related operations
    Inherits from BaseClient for HTTP functionality
    """
    
    def test_connection(self) -> bool:
        """Test API connection by attempting to authenticate"""
        try:
            # Simple test endpoint - try to get a non-existent article to test auth
            self.get("/api/articles/test-connection")
            return True
        except Exception as error:
            # If we get 404, auth is working but article doesn't exist (expected)
            if hasattr(error, 'status_code') and error.status_code == 404:
                return True
            # If we get 401/403, auth is failing
            if hasattr(error, 'status_code') and error.status_code in (401, 403):
                return False
            # Other errors might indicate network issues but auth could be fine
            return False
    
    def generate_articles(self, request: ArticleGenerationRequest) -> ArticleGenerationResponse:
        """
        Generate new articles from keywords
        Supports single keyword or bulk generation
        """
        self._validate_generation_request(request)
        
        try:
            # Convert dataclass to dict for API call
            request_data = {
                "targetKeyword": request.target_keyword,
            }
            
            # Add optional fields if present
            if request.gpt_version:
                request_data["gptVersion"] = request.gpt_version
            if request.integration_id:
                request_data["integrationId"] = request.integration_id
            if request.preset_id:
                request_data["presetId"] = request.preset_id
            if request.project_name:
                request_data["projectName"] = request.project_name
            if request.personalisation_name:
                request_data["personalisationName"] = request.personalisation_name
            if request.additional_params:
                request_data.update(request.additional_params)
            
            response_data = self.post("/api/articles", body=request_data)
            
            if response_data.get("error"):
                raise Exception(response_data["error"])
            
            # Convert response to dataclass
            return ArticleGenerationResponse(
                project_id=response_data["projectId"],
                message=response_data["message"],
                article_id=response_data.get("articleId"),
                article_ids=response_data.get("articleIds"),
                processing_info=response_data.get("processingInfo"),
                error=response_data.get("error")
            )
            
        except Exception as error:
            raise error
    
    def generate_article(
        self,
        target_keyword: str,
        options: Optional[Dict[str, Any]] = None
    ) -> Dict[str, str]:
        """
        Generate a single article from keyword
        Convenience method for single article generation
        """
        options = options or {}
        
        request = ArticleGenerationRequest(
            target_keyword=target_keyword,
            gpt_version=options.get("gpt_version"),
            integration_id=options.get("integration_id"),
            preset_id=options.get("preset_id"),
            project_name=options.get("project_name"),
            personalisation_name=options.get("personalisation_name"),
            additional_params=options.get("additional_params")
        )
        
        response = self.generate_articles(request)
        
        # Handle both single articleId and articleIds array formats
        article_id = None
        if response.article_id:
            article_id = response.article_id
        elif response.article_ids and len(response.article_ids) > 0:
            article_id = response.article_ids[0]
        else:
            raise Exception("No articles were generated")
        
        return {
            "articleId": article_id,
            "projectId": response.project_id,
            "message": response.message
        }
    
    def generate_bulk_articles(
        self,
        keywords: List[str],
        options: Optional[Dict[str, Any]] = None
    ) -> BulkOperationResult:
        """
        Generate multiple articles in bulk
        Handles array of keywords with shared configuration
        """
        if not keywords or len(keywords) == 0:
            raise ValidationError("Keywords array cannot be empty")
        
        options = options or {}
        successful = []
        failed = []
        
        # Generate articles for all keywords at once
        try:
            request = ArticleGenerationRequest(
                target_keyword=keywords,
                gpt_version=options.get("gpt_version"),
                integration_id=options.get("integration_id"),
                preset_id=options.get("preset_id"),
                project_name=options.get("project_name"),
                personalisation_name=options.get("personalisation_name"),
                additional_params=options.get("additional_params")
            )
            
            response = self.generate_articles(request)
            
            # Map article IDs back to keywords
            if response.article_ids:
                for index, article_id in enumerate(response.article_ids):
                    successful.append({
                        "article_id": article_id,
                        "keyword": keywords[index] if index < len(keywords) else f"keyword_{index}"
                    })
        
        except Exception as error:
            # If bulk generation fails, mark all as failed
            for keyword in keywords:
                failed.append({
                    "item": keyword,
                    "error": str(error) or "Generation failed"
                })
        
        return BulkOperationResult(
            successful=successful,
            failed=failed,
            total=len(keywords),
            success_count=len(successful),
            failure_count=len(failed)
        )
    
    def get_article(self, article_id: str) -> Article:
        """
        Retrieve article by ID
        Returns complete article with content and metadata
        """
        if not article_id or not isinstance(article_id, str):
            raise ValidationError("Article ID is required and must be a string")
        
        try:
            article_data = self.get(f"/api/articles/{article_id}")
            
            # Handle JSON string response
            if isinstance(article_data, str):
                import json
                article_data = json.loads(article_data)
            
            # Map API response fields to our expected format
            return Article(
                id=article_data["id"],
                organization_id=article_data["organization_id"],
                user_id=article_data["user_id"],
                project_id=article_data.get("project_id", ""),
                target_keyword=article_data.get("config", {}).get("targetKeyword", ""),
                status=article_data["status"],
                created_at=article_data["created_at"],
                updated_at=article_data.get("updatedAt", article_data["created_at"]),
                article_html=article_data.get("output"),
                content=article_data.get("output"),
                html=article_data.get("output"),
                title=article_data.get("extra_data", {}).get("targetArticleTopic"),
                metadata=article_data.get("config"),
                completed_at=article_data.get("updatedAt"),
                error_message=article_data.get("error")
            )
            
        except Exception as error:
            raise error
    
    def get_articles(self, article_ids: List[str]) -> BulkOperationResult:
        """
        Get multiple articles by IDs
        Handles bulk retrieval with error handling for individual articles
        """
        if not article_ids or len(article_ids) == 0:
            raise ValidationError("Article IDs array cannot be empty")
        
        successful = []
        failed = []
        
        # Process articles sequentially (could be made concurrent with asyncio)
        for article_id in article_ids:
            try:
                article = self.get_article(article_id)
                successful.append(article)
            except Exception as error:
                failed.append({
                    "item": article_id,
                    "error": str(error) or "Failed to retrieve article"
                })
        
        return BulkOperationResult(
            successful=successful,
            failed=failed,
            total=len(article_ids),
            success_count=len(successful),
            failure_count=len(failed)
        )
    
    def is_article_complete(self, article_id: str) -> bool:
        """
        Check if article generation is complete
        Returns true if article has content, false if still processing
        """
        try:
            article = self.get_article(article_id)
            return bool(
                (article.article_html or article.content) and
                article.status not in ("queued", "processing")
            )
        except Exception as error:
            if hasattr(error, 'status_code') and error.status_code == 404:
                return False
            raise error
    
    def get_article_status(self, article_id: str) -> Dict[str, Any]:
        """
        Get article generation status
        Returns simplified status information
        """
        try:
            article = self.get_article(article_id)
            
            return {
                "id": article.id,
                "status": article.status,
                "isComplete": self._is_status_complete(article.status),
                "hasError": article.status == "failed",
                "errorMessage": article.error_message
            }
        except Exception as error:
            return {
                "id": article_id,
                "status": "unknown",
                "isComplete": False,
                "hasError": True,
                "errorMessage": str(error) or "Failed to get status"
            }
    
    def _validate_generation_request(self, request: ArticleGenerationRequest) -> None:
        """Private helper to validate generation requests"""
        if not request.target_keyword:
            raise ValidationError("Target keyword is required")
        
        if isinstance(request.target_keyword, str):
            if not request.target_keyword.strip():
                raise ValidationError("Target keyword cannot be empty")
        elif isinstance(request.target_keyword, list):
            if len(request.target_keyword) == 0:
                raise ValidationError("Target keywords array cannot be empty")
            
            invalid_keywords = [
                keyword for keyword in request.target_keyword
                if not keyword or not isinstance(keyword, str) or not keyword.strip()
            ]
            
            if invalid_keywords:
                raise ValidationError("All keywords must be non-empty strings")
        else:
            raise ValidationError("Target keyword must be a string or list of strings")
    
    def _is_status_complete(self, status: str) -> bool:
        """Private helper to check if status indicates completion"""
        return status.lower() in ["completed", "failed", "cancelled", "finished"]