"""
Core data models for SemanticPen SDK
"""

from typing import Any, Dict, List, Optional, Union
from dataclasses import dataclass
from .enums import ArticleStatusEnum


# ============================================================================
# AUTHENTICATION MODELS
# ============================================================================

@dataclass
class SemanticPenConfig:
    """Configuration for SemanticPen client initialization"""
    api_key: str
    base_url: str = "https://www.semanticpen.com"
    timeout: int = 30000
    debug: bool = False


@dataclass 
class AuthResult:
    """Authentication response"""
    user_id: str
    organization_id: str
    success: bool


# ============================================================================
# ARTICLE GENERATION MODELS
# ============================================================================

@dataclass
class ArticleGenerationRequest:
    """Article generation request parameters"""
    target_keyword: Union[str, List[str]]
    gpt_version: Optional[str] = None
    integration_id: Optional[str] = None
    preset_id: Optional[str] = None
    project_name: Optional[str] = None
    personalisation_name: Optional[str] = None
    additional_params: Optional[Dict[str, Any]] = None


@dataclass
class ArticleGenerationResponse:
    """Article generation response"""
    project_id: str
    message: str
    article_id: Optional[str] = None
    article_ids: Optional[List[str]] = None
    processing_info: Optional[str] = None
    error: Optional[str] = None


# ============================================================================
# ARTICLE RETRIEVAL MODELS  
# ============================================================================

@dataclass
class Article:
    """Complete article with all fields"""
    id: str
    organization_id: str
    user_id: str
    project_id: str
    target_keyword: str
    status: ArticleStatusEnum
    created_at: str
    updated_at: str
    article_html: Optional[str] = None
    content: Optional[str] = None
    html: Optional[str] = None
    title: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    completed_at: Optional[str] = None
    error_message: Optional[str] = None


@dataclass
class ArticleStatusInfo:
    """Simplified article status information"""
    id: str
    status: str
    is_complete: bool
    has_error: bool
    error_message: Optional[str] = None
    progress: Optional[int] = None
    estimated_completion_time: Optional[str] = None


# ============================================================================
# ERROR MODELS
# ============================================================================

@dataclass
class ApiError:
    """Standard API error response"""
    error: str
    code: Optional[str] = None
    details: Optional[Dict[str, Any]] = None


# ============================================================================
# UTILITY MODELS
# ============================================================================

@dataclass
class RequestOptions:
    """Request options for API calls"""
    method: str
    headers: Optional[Dict[str, str]] = None
    body: Optional[Any] = None
    timeout: Optional[int] = None


@dataclass
class PollingConfig:
    """Polling configuration"""
    interval: int
    max_attempts: int
    backoff_multiplier: float = 1.0


@dataclass
class BulkOperationResult:
    """Bulk operation result"""
    successful: List[Any]
    failed: List[Dict[str, Any]]
    total: int
    success_count: int
    failure_count: int