"""
Status polling utility for article generation completion
"""

import time
from typing import Any, Dict, List, Optional, Callable

from ..types.models import PollingConfig, Article
from ..services.article_service import ArticleService


class StatusPoller:
    """
    Utility class for polling article generation status
    Implements exponential backoff and timeout handling
    """
    
    def __init__(self, article_service: ArticleService):
        self.article_service = article_service
        self.default_config = PollingConfig(
            interval=5000,  # 5 seconds
            max_attempts=60,  # 5 minutes total
            backoff_multiplier=1.1
        )
    
    def poll_until_complete(
        self,
        article_id: str,
        polling_config: Optional[PollingConfig] = None,
        callbacks: Optional[Dict[str, Callable]] = None
    ) -> Dict[str, Any]:
        """
        Poll until article generation is complete
        Returns success/failure result with article data
        """
        config = polling_config or self.default_config
        callbacks = callbacks or {}
        
        interval = config.interval / 1000.0  # Convert ms to seconds
        attempts = 0
        
        while attempts < config.max_attempts:
            try:
                # Check if article is complete
                if self.article_service.is_article_complete(article_id):
                    article = self.article_service.get_article(article_id)
                    
                    if callbacks.get("on_complete"):
                        callbacks["on_complete"](article)
                    
                    return {
                        "success": True,
                        "article": article,
                        "attempts": attempts + 1
                    }
                
                # Get status for progress callback
                status = self.article_service.get_article_status(article_id)
                
                if callbacks.get("on_progress"):
                    callbacks["on_progress"](status, attempts + 1, config.max_attempts)
                
                # Check for error status
                if status.get("hasError"):
                    error_msg = status.get("errorMessage", "Article generation failed")
                    
                    if callbacks.get("on_error"):
                        callbacks["on_error"](error_msg)
                    
                    return {
                        "success": False,
                        "error": error_msg,
                        "attempts": attempts + 1
                    }
                
                # Sleep before next attempt
                time.sleep(interval)
                
                # Apply backoff multiplier
                interval *= config.backoff_multiplier
                attempts += 1
                
            except Exception as error:
                if callbacks.get("on_error"):
                    callbacks["on_error"](str(error))
                
                return {
                    "success": False,
                    "error": str(error),
                    "attempts": attempts + 1
                }
        
        # Timeout reached
        timeout_error = f"Polling timeout after {config.max_attempts} attempts"
        
        if callbacks.get("on_timeout"):
            callbacks["on_timeout"](timeout_error)
        
        return {
            "success": False,
            "error": timeout_error,
            "attempts": attempts
        }
    
    def wait_for_generation_complete(
        self,
        article_ids: List[str],
        polling_config: Optional[PollingConfig] = None,
        on_progress: Optional[Callable[[int, int, List[Article]], None]] = None
    ) -> Dict[str, Any]:
        """
        Wait for multiple articles to complete
        Handles bulk waiting with progress tracking
        """
        config = polling_config or self.default_config
        start_time = time.time()
        
        completed = []
        failed = []
        pending_ids = list(article_ids)
        
        interval = config.interval / 1000.0
        attempts = 0
        
        while pending_ids and attempts < config.max_attempts:
            completed_this_round = []
            
            for article_id in pending_ids[:]:  # Copy list to modify during iteration
                try:
                    if self.article_service.is_article_complete(article_id):
                        article = self.article_service.get_article(article_id)
                        
                        # Check if article has error
                        status = self.article_service.get_article_status(article_id)
                        if status.get("hasError"):
                            failed.append({
                                "article_id": article_id,
                                "error": status.get("errorMessage", "Generation failed")
                            })
                        else:
                            completed.append(article)
                            completed_this_round.append(article)
                        
                        pending_ids.remove(article_id)
                
                except Exception as error:
                    failed.append({
                        "article_id": article_id,
                        "error": str(error)
                    })
                    pending_ids.remove(article_id)
            
            # Call progress callback
            if on_progress:
                on_progress(len(completed), len(article_ids), completed_this_round)
            
            # If we still have pending articles, wait before next check
            if pending_ids:
                time.sleep(interval)
                interval *= config.backoff_multiplier
                attempts += 1
        
        # Mark remaining pending articles as failed due to timeout
        for article_id in pending_ids:
            failed.append({
                "article_id": article_id,
                "error": f"Timeout after {config.max_attempts} attempts"
            })
        
        return {
            "completed": completed,
            "failed": failed,
            "total_time": time.time() - start_time
        }
    
    def get_multiple_status(self, article_ids: List[str]) -> Dict[str, List[Any]]:
        """
        Get status of multiple articles without polling
        One-time status check for bulk articles
        """
        completed = []
        pending = []
        failed = []
        not_found = []
        
        for article_id in article_ids:
            try:
                status = self.article_service.get_article_status(article_id)
                
                if status.get("hasError"):
                    failed.append(status)
                elif status.get("isComplete"):
                    completed.append(status)
                else:
                    pending.append(status)
            
            except Exception as error:
                if hasattr(error, 'status_code') and error.status_code == 404:
                    not_found.append(article_id)
                else:
                    failed.append({
                        "id": article_id,
                        "status": "error",
                        "isComplete": False,
                        "hasError": True,
                        "errorMessage": str(error)
                    })
        
        return {
            "completed": completed,
            "pending": pending,
            "failed": failed,
            "notFound": not_found
        }