"""
Tests for SemanticPenClient
"""

import pytest
from unittest.mock import Mock, patch
from semanticpen import (
    SemanticPenClient,
    SemanticPenConfig,
    ValidationError,
    AuthenticationError
)


class TestSemanticPenClient:
    """Test cases for SemanticPenClient"""
    
    def test_create_client_with_api_key(self):
        """Test creating client with API key"""
        client = SemanticPenClient.create("test-api-key")
        
        assert client is not None
        config = client.get_config()
        assert config.api_key == "test-api-key"
        assert config.base_url == "https://www.semanticpen.com"
        assert config.timeout == 30000
        assert config.debug is False
    
    def test_create_client_with_options(self):
        """Test creating client with options"""
        client = SemanticPenClient.create(
            "test-api-key",
            base_url="https://custom.api.com",
            timeout=60000,
            debug=True
        )
        
        config = client.get_config()
        assert config.api_key == "test-api-key"
        assert config.base_url == "https://custom.api.com"
        assert config.timeout == 60000
        assert config.debug is True
    
    def test_create_with_config(self):
        """Test creating client with configuration object"""
        config = SemanticPenConfig(
            api_key="test-key",
            base_url="https://test.com",
            timeout=45000,
            debug=True
        )
        
        client = SemanticPenClient.create_with_config(config)
        retrieved_config = client.get_config()
        
        assert retrieved_config.api_key == "test-key"
        assert retrieved_config.base_url == "https://test.com"
        assert retrieved_config.timeout == 45000
        assert retrieved_config.debug is True
    
    def test_validate_keyword_valid(self):
        """Test keyword validation with valid inputs"""
        assert SemanticPenClient.validate_keyword("valid keyword") is True
        assert SemanticPenClient.validate_keyword("  spaced keyword  ") is True
        assert SemanticPenClient.validate_keyword("single") is True
    
    def test_validate_keyword_invalid(self):
        """Test keyword validation with invalid inputs"""
        assert SemanticPenClient.validate_keyword("") is False
        assert SemanticPenClient.validate_keyword("   ") is False
        assert SemanticPenClient.validate_keyword(None) is False
        assert SemanticPenClient.validate_keyword(123) is False
    
    def test_validate_keywords_valid(self):
        """Test keywords validation with valid inputs"""
        result = SemanticPenClient.validate_keywords(["keyword1", "keyword2"])
        assert result["valid"] is True
        assert len(result["errors"]) == 0
        
        result = SemanticPenClient.validate_keywords(["single"])
        assert result["valid"] is True
        assert len(result["errors"]) == 0
    
    def test_validate_keywords_invalid(self):
        """Test keywords validation with invalid inputs"""
        # Empty array
        result = SemanticPenClient.validate_keywords([])
        assert result["valid"] is False
        assert "cannot be empty" in result["errors"][0]
        
        # Not an array
        result = SemanticPenClient.validate_keywords("not an array")
        assert result["valid"] is False
        assert "must be a list" in result["errors"][0]
        
        # Invalid keywords in array
        result = SemanticPenClient.validate_keywords(["valid", "", "also valid"])
        assert result["valid"] is False
        assert "Invalid keyword at index 1" in result["errors"][0]
    
    @patch('semanticpen.core.base_client.BaseClient.get')
    def test_test_connection_success(self, mock_get):
        """Test successful connection test"""
        # Mock 404 response (expected for test endpoint)
        mock_error = Exception()
        mock_error.status_code = 404
        mock_get.side_effect = mock_error
        
        client = SemanticPenClient.create("test-api-key")
        result = client.test_connection()
        
        assert result is True
        mock_get.assert_called_once_with("/api/articles/test-connection")
    
    @patch('semanticpen.core.base_client.BaseClient.get')
    def test_test_connection_auth_failure(self, mock_get):
        """Test connection test with auth failure"""
        # Mock 401 response (auth failure)
        mock_error = Exception()
        mock_error.status_code = 401
        mock_get.side_effect = mock_error
        
        client = SemanticPenClient.create("invalid-api-key")
        result = client.test_connection()
        
        assert result is False
    
    @patch('semanticpen.services.article_service.ArticleService.generate_article')
    def test_generate_article_success(self, mock_generate):
        """Test successful article generation"""
        # Mock successful response
        mock_generate.return_value = {
            "articleId": "test-article-id",
            "projectId": "test-project-id",
            "message": "Article generation started"
        }
        
        client = SemanticPenClient.create("test-api-key")
        result = client.generate_article("test keyword")
        
        assert result["articleId"] == "test-article-id"
        assert result["projectId"] == "test-project-id"
        assert result["message"] == "Article generation started"
        
        mock_generate.assert_called_once_with("test keyword", {})
    
    @patch('semanticpen.services.article_service.ArticleService.generate_article')
    def test_generate_article_with_options(self, mock_generate):
        """Test article generation with options"""
        mock_generate.return_value = {
            "articleId": "test-id",
            "projectId": "test-project",
            "message": "Started"
        }
        
        client = SemanticPenClient.create("test-api-key")
        result = client.generate_article(
            "test keyword",
            gpt_version="gpt-4",
            project_name="Test Project"
        )
        
        # Verify the options were passed through
        mock_generate.assert_called_once_with(
            "test keyword",
            {"gpt_version": "gpt-4", "project_name": "Test Project"}
        )
    
    def test_create_request_builder(self):
        """Test creating request builder"""
        client = SemanticPenClient.create("test-api-key")
        builder = client.create_request_builder()
        
        assert builder is not None
        assert hasattr(builder, 'keyword')
        assert hasattr(builder, 'gpt_version')
        assert hasattr(builder, 'generate')


class TestSemanticPenConfig:
    """Test cases for SemanticPenConfig"""
    
    def test_config_creation(self):
        """Test configuration object creation"""
        config = SemanticPenConfig(
            api_key="test-key",
            base_url="https://test.com",
            timeout=60000,
            debug=True
        )
        
        assert config.api_key == "test-key"
        assert config.base_url == "https://test.com"
        assert config.timeout == 60000
        assert config.debug is True
    
    def test_config_defaults(self):
        """Test configuration defaults"""
        config = SemanticPenConfig(api_key="test-key")
        
        assert config.api_key == "test-key"
        assert config.base_url == "https://www.semanticpen.com"
        assert config.timeout == 30000
        assert config.debug is False


if __name__ == "__main__":
    pytest.main([__file__])