"""
Tests for error handling classes
"""

import pytest
import json
from datetime import datetime
from semanticpen.errors import (
    SemanticPenError,
    AuthenticationError,
    ValidationError,
    RateLimitError,
    NetworkError,
    ApiError,
    TimeoutError,
    UnknownError,
    ErrorFactory
)
from semanticpen.types.enums import ErrorType


class TestSemanticPenErrors:
    """Test cases for SemanticPen error classes"""
    
    def test_authentication_error(self):
        """Test AuthenticationError"""
        error = AuthenticationError("Invalid API key")
        
        assert error.message == "Invalid API key"
        assert error.type == ErrorType.AUTHENTICATION_ERROR
        assert isinstance(error.timestamp, datetime)
        assert "Authentication Error: Invalid API key" in str(error)
    
    def test_validation_error(self):
        """Test ValidationError"""
        error = ValidationError(
            "Invalid input",
            field="keyword",
            value="",
            details={"min_length": 1}
        )
        
        assert error.message == "Invalid input"
        assert error.field == "keyword"
        assert error.value == ""
        assert error.details["min_length"] == 1
        assert error.type == ErrorType.VALIDATION_ERROR
        assert "field 'keyword'" in str(error)
    
    def test_validation_error_without_field(self):
        """Test ValidationError without field"""
        error = ValidationError("General validation error")
        
        assert error.field is None
        assert "Validation Error: General validation error" in str(error)
    
    def test_rate_limit_error(self):
        """Test RateLimitError"""
        error = RateLimitError(
            "Rate limit exceeded",
            retry_after=60,
            details={"daily_limit": 1000, "current_usage": 1001}
        )
        
        assert error.message == "Rate limit exceeded"
        assert error.retry_after == 60
        assert error.daily_limit == 1000
        assert error.current_usage == 1001
        assert error.type == ErrorType.RATE_LIMIT_ERROR
        assert "Retry after 60 seconds" in str(error)
        assert "Usage: 1001/1000" in str(error)
    
    def test_network_error(self):
        """Test NetworkError"""
        error = NetworkError(
            "Connection failed",
            status_code=503,
            endpoint="/api/articles"
        )
        
        assert error.message == "Connection failed"
        assert error.status_code == 503
        assert error.endpoint == "/api/articles"
        assert error.type == ErrorType.NETWORK_ERROR
        assert "Status: 503" in str(error)
        assert "Endpoint: /api/articles" in str(error)
    
    def test_api_error(self):
        """Test ApiError"""
        response_data = {"error": "Article not found", "code": "NOT_FOUND"}
        error = ApiError(
            "Article not found",
            status_code=404,
            endpoint="/api/articles/123",
            response=response_data
        )
        
        assert error.message == "Article not found"
        assert error.status_code == 404
        assert error.endpoint == "/api/articles/123"
        assert error.response == response_data
        assert error.type == ErrorType.API_ERROR
        assert "[404] at /api/articles/123" in str(error)
    
    def test_timeout_error(self):
        """Test TimeoutError"""
        error = TimeoutError(
            "Request timed out",
            timeout_ms=30000,
            operation="article generation"
        )
        
        assert error.message == "Request timed out"
        assert error.timeout_ms == 30000
        assert error.operation == "article generation"
        assert error.type == ErrorType.TIMEOUT_ERROR
        assert "(30000ms)" in str(error)
        assert "during article generation" in str(error)
    
    def test_unknown_error(self):
        """Test UnknownError"""
        original = ValueError("Original error")
        error = UnknownError(
            "Something went wrong",
            original_error=original
        )
        
        assert error.message == "Something went wrong"
        assert error.original_error == original
        assert error.type == ErrorType.UNKNOWN_ERROR
        assert "Original: Original error" in str(error)
    
    def test_error_to_dict(self):
        """Test error serialization to dictionary"""
        error = ValidationError("Test error", field="test", details={"key": "value"})
        error_dict = error.to_dict()
        
        assert error_dict["name"] == "ValidationError"
        assert error_dict["message"] == "Test error"
        assert error_dict["type"] == ErrorType.VALIDATION_ERROR.value
        assert "timestamp" in error_dict
        assert error_dict["details"]["key"] == "value"
    
    def test_error_to_json(self):
        """Test error serialization to JSON"""
        error = AuthenticationError("Auth failed")
        json_str = error.to_json()
        
        # Should be valid JSON
        data = json.loads(json_str)
        assert data["name"] == "AuthenticationError"
        assert data["message"] == "Auth failed"
        assert data["type"] == ErrorType.AUTHENTICATION_ERROR.value


class TestErrorFactory:
    """Test cases for ErrorFactory"""
    
    def test_from_http_response_401(self):
        """Test creating error from 401 response"""
        # Mock response object
        response = MockResponse(401, "Unauthorized")
        response_data = {"error": "Invalid token"}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, AuthenticationError)
        assert error.message == "Invalid token"
        assert error.details["status_code"] == 401
        assert error.details["endpoint"] == "/api/test"
    
    def test_from_http_response_400(self):
        """Test creating error from 400 response"""
        response = MockResponse(400, "Bad Request")
        response_data = {"error": "Invalid input"}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, ValidationError)
        assert error.message == "Invalid input"
    
    def test_from_http_response_429(self):
        """Test creating error from 429 response"""
        response = MockResponse(429, "Too Many Requests")
        response_data = {"error": "Rate limit exceeded", "retry_after": 30}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, RateLimitError)
        assert error.message == "Rate limit exceeded"
        assert error.retry_after == 30
    
    def test_from_http_response_404(self):
        """Test creating error from 404 response"""
        response = MockResponse(404, "Not Found")
        response_data = {"error": "Resource not found"}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, ApiError)
        assert error.message == "Resource not found"
        assert error.status_code == 404
    
    def test_from_http_response_500(self):
        """Test creating error from 500 response"""
        response = MockResponse(500, "Internal Server Error")
        response_data = {"error": "Server error"}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, NetworkError)
        assert error.message == "Server error"
        assert error.status_code == 500
    
    def test_from_http_response_unknown_status(self):
        """Test creating error from unknown status code"""
        response = MockResponse(418, "I'm a teapot")
        response_data = {"error": "Teapot error"}
        
        error = ErrorFactory.from_http_response(response, "/api/test", response_data)
        
        assert isinstance(error, ApiError)
        assert error.message == "Teapot error"
        assert error.status_code == 418
    
    def test_from_exception_semantic_pen_error(self):
        """Test from_exception with SemanticPenError"""
        original_error = ValidationError("Original error")
        
        result = ErrorFactory.from_exception(original_error, "test context")
        
        # Should return the same error
        assert result is original_error
    
    def test_from_exception_type_error(self):
        """Test from_exception with TypeError"""
        original_error = TypeError("Type error")
        
        result = ErrorFactory.from_exception(original_error, "test context")
        
        assert isinstance(result, ValidationError)
        assert "Type error" in result.message
        assert result.details["context"] == "test context"
    
    def test_from_exception_connection_error(self):
        """Test from_exception with ConnectionError"""
        original_error = ConnectionError("Connection failed")
        
        result = ErrorFactory.from_exception(original_error, "test context")
        
        assert isinstance(result, NetworkError)
        assert "Connection failed" in result.message
    
    def test_from_exception_unknown_error(self):
        """Test from_exception with unknown error"""
        original_error = RuntimeError("Runtime error")
        
        result = ErrorFactory.from_exception(original_error, "test context")
        
        assert isinstance(result, UnknownError)
        assert "Runtime error" in result.message
        assert result.original_error == original_error


class MockResponse:
    """Mock response object for testing"""
    
    def __init__(self, status_code, reason):
        self.status_code = status_code
        self.reason = reason


if __name__ == "__main__":
    pytest.main([__file__])